/*+++*
 *  title:	MailboxInspector.m
 *  abstract:	NEXTSTEP Workspace Manager Inspector for Mail.app ".mbox" files.
 *  author:	T.R.Hageman, The Netherlands
 *  created:	May 1996
 *  modified:	(see RCS Log at end)
 *  copyleft:
 *
 *		Copyright (C) 1996-1998  Tom R. Hageman.
 *
 *	This is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This software is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this software; see the file COPYING.  If not, write to
 *	the Free Software Foundation, 59 Temple Place - Suite 330,
 *	Boston, MA 02111-1307, USA.
 *
 *  description:
 *
 *	XXX TODO differentiate between unloaded and corrupt TOC.
 *---*/

static const char *const RCSid = ((void)&RCSid, /* to avoid `unused' warning. */
	"@(#)MailboxInspector.m,v 1.8 1999/03/01 21:04:40 tom Exp");

#define VERSION	"1.3"

#ifndef DEBUG
#   define DEBUG 0
#endif

#import "MailboxInspector.h"
#import "MailboxSummary.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "mailtoc.h"

// Icon names

#define ICON_NEWMAIL	"newmail2"
#define ICON_INVALID	"Envelope+Triangle"

// Localized strings

#define OPENBUTTON	NXLocalizedStringFromTableInBundle(NULL, bundle, "Open", NULL, okbutton label)
#define INCORPBUTTON	NXLocalizedStringFromTableInBundle(NULL, bundle, "Incorporate", NULL, revertbutton label)

#define NO_NEW_MESSAGES	NXLocalizedStringFromTableInBundle(NULL, bundle, "", NULL, newMailTextField: No new messages)
#define ONE_NEW_MESSAGE	NXLocalizedStringFromTableInBundle(NULL, bundle, "1 new message", NULL, newMailTextField: 1 new message)
#define N_NEW_MESSAGES	NXLocalizedStringFromTableInBundle(NULL, bundle, "%lu new messages", NULL, newMailTextField: %lu new messages format string)
#define CANNOT_READ_TOC	NXLocalizedStringFromTableInBundle(NULL, bundle, "Unable to read Table of Contents", NULL, newMailTextField: Cannot read Table of Contents)
#define INDEX_SIZE	NXLocalizedStringFromTableInBundle(NULL, bundle, "(index: %s)", NULL, indexSizeField: (index: %s))
#define TOC_SIZE	NXLocalizedStringFromTableInBundle(NULL, bundle, "(table of contents: %s)", NULL, tocSizeField: (table of contents: %s))

#define KBYTES		NXLocalizedStringFromTableInBundle(NULL, bundle, "KB", NULL, KiloBytes)
#define MBYTES		NXLocalizedStringFromTableInBundle(NULL, bundle, "MB", NULL, MegaBytes)

#define LOCALIZE(s)	NXLoadLocalized\
StringFromTableInBundle(NULL, bundle, s, NULL)
// (line continuation to keep "genstrings" from breaking.)


@implementation MailboxInspector

- initWithNibName:(const char *)nibname
{
	char path[MAXPATHLEN+1];

	bundle = [NXBundle bundleForClass:[self class]];

	if ([bundle getPath:path forResource:nibname ofType:"nib"] &&
	    [NXApp loadNibFile:path owner:self])
	{
		// Set version field.
		[inspectorVersionField setStringValue:VERSION];
	}
	else
	{
		fprintf(stderr, "Couldn't load %s.nib\n", nibname);
		[self free];
		self = nil;
	}
	return self;
}

+ new
{
	static MailboxInspector *instance;
	
	if (instance == nil)
	{
		instance = [[super new] initWithNibName:[self name]];
	}
	return instance;
}

- showInfo:sender
{
	if (infoPanel == nil)
	{
		char path[MAXPATHLEN+1];

		if ([bundle getPath:path forResource:"Info" ofType:"nib"] &&
		    [NXApp loadNibFile:path owner:self])
		{
			[infoVersionField setStringValue:[inspectorVersionField stringValue]];
		}
	}
	[infoPanel makeKeyAndOrderFront:sender];
	return self;
}

- revert:sender
{
	char path[MAXPATHLEN+1];

	[super revert:sender];

	if ([self selectionCount] != 1) return nil;

	if (sender == [self revertButton])
	{
		[mailbox incorporatePendingAppnmail:YES];
	}

	[self selectionPathsInto:path separator:'\0'];

	if (mailbox == nil || strcmp(path, [mailbox name]) != 0)
	{
		[mailbox free];

		mailbox = [[EnhanceMailboxSummary allocFromZone:[self zone]] initWithName:path];
		if (mailbox == nil) return nil;

		[mailbox setCollectAll:YES];
	}

	[self load];

	[[[self revertButton] setTitle:INCORPBUTTON] setEnabled:[mailbox isLoaded] && [mailbox hasPendingAppnmail]];
	[[[self okButton] setTitle:OPENBUTTON] setEnabled:[mailbox isLoaded]];

	return self;
}

- ok:sender
{
	[self perform:@selector(open:) with:sender afterDelay:0 cancelPrevious:NO];
	[super ok:sender];
	return self;
}

- (void)load
{
	unsigned long counts[EMS_NUMTAGS];
	unsigned long sizes[EMS_NUMTAGS];
	unsigned long attachCounts[EMS_NUMTAGS];
	unsigned long attachSizes[EMS_NUMTAGS];
	unsigned long tocSize, indexSize;
	BOOL valid;
	BOOL indexed;

	char buf[BUFSIZ];
	char sizeBuf[20];
	int tag;

	/* (re-)Load mailbox summary, and copy results... */

	[mailbox load];

	valid = [mailbox isLoaded];

	for (tag = 0;  tag < EMS_NUMTAGS;  tag++)
	{
		counts[tag]	  = [mailbox count:tag];
		sizes[tag]	  = [mailbox size:tag];
		attachCounts[tag] = [mailbox attachCount:tag];
		attachSizes[tag]  = [mailbox attachSize:tag];
	}

	indexed = [mailbox hasIndex];

	/* Fill in the inspector fields. */

	[[self window] disableFlushWindow];

	[indexTagButton setState:(indexed ? 0 : 1)];
	[newMailIconButton setAltIcon:(valid ? ICON_NEWMAIL : ICON_INVALID)];
	[newMailIconButton setState:(!valid || counts[EMS_TAG_NEW] > 0)];
	[newMailTextField setStringValue:
	 (valid ? (counts[EMS_TAG_NEW] == 0 ? NO_NEW_MESSAGES :
		   counts[EMS_TAG_NEW] == 1 ? ONE_NEW_MESSAGE :
		   (sprintf(buf, N_NEW_MESSAGES, counts[EMS_TAG_NEW]), buf)) :
	  CANNOT_READ_TOC)];
	[pendingAppnmailIconButton setState:([mailbox hasPendingAppnmail] ? 0 : 1)];
	for (tag = 0;  tag < EMS_NUMTAGS;  tag++)
	{
#define FORMATCOUNT(count) \
		((count) ? (sprintf(sizeBuf, "%lu", (count)), sizeBuf) : "")
#define FORMATSIZE(size) \
		([self formatSize:(size) inBuf:sizeBuf])

		[[messageCountMatrix findCellWithTag:tag]
		 setStringValue:FORMATCOUNT(counts[tag])];
		[[messageSizeMatrix findCellWithTag:tag]
		 setStringValue:FORMATSIZE(sizes[tag])];
		[[attachmentCountMatrix findCellWithTag:tag]
		 setStringValue:FORMATCOUNT(attachCounts[tag])];
		[[attachmentSizeMatrix findCellWithTag:tag]
		 setStringValue:FORMATSIZE(attachSizes[tag])];
#undef FORMATCOUNT
#undef FORMATSIZE
	}
	[indexSizeField setStringValue:
	 (indexed && (indexSize = [mailbox indexSize]) > 0 ?
	  (sprintf(buf, INDEX_SIZE, [self formatSize:indexSize inBuf:sizeBuf]), buf) : "")];
	[tocSizeField setStringValue:
	 ((tocSize = [mailbox tableOfContentsSize]) > 0 ?
	  (sprintf(buf, TOC_SIZE, [self formatSize:tocSize inBuf:sizeBuf]), buf) : "")];

	[[self window] reenableFlushWindow];
	[[self window] flushWindowIfNeeded];
}


// Support methods

- (const char *)formatSize:(unsigned long)size inBuf:(char *)buf
{
#define KB	1024
#define MB	(KB * 1024)

	if (size == 0)
	{
		strcpy(buf, "");
	}
	else if (size < 1000*KB - KB/2)
	{
		sprintf(buf, "%.*f%s",
			//(size < 10*KB - KB/2 ? 2 : size < 100*KB - KB/2 ? 1 : 0),
			(size < 10*KB - KB/2 ? 1 : 0),
			(double) size / KB, KBYTES);
	}
	else
	{
		sprintf(buf, "%.*f%s",
			//(size < 10*MB - MB/2 ? 2 : size < 100*MB - MB/2 ? 1 : 0),
			(size < 10*MB - MB/2 ? 1 : 0),
			(double) size / MB, MBYTES);
	}
	return buf;
#undef KB
#undef MB
}


static void openInWorkspace(const char *filename)
{
	// Indirect approach to circumvent Workspace deadlock/timeout.
	char command[14+3*MAXPATHLEN+1];
	const char *s;
	char *d = command;

	for (s = "exec open '"; *s; ) *d++ = *s++;
	// Escape single quote characters.
	for (s = filename; *s; )
	{
		if ((*d++ = *s++) == '\'')
		{
			*d++ = '\\', *d++ = '\'', *d++ = '\'';
		}
	}
	for (s = "'&"; *d++ = *s++; ) ;
	system(command);
}

- open:sender
{
	openInWorkspace([mailbox name]);
	return self;
}

@end // MailboxInspector

//======================================================================
// MailboxInspector.m,v
// Revision 1.8  1999/03/01 21:04:40  tom
// MailboxInspector:
// (VERSION): updated to 1.3.
//
// README.rtf:
// - prepare for release 1.3.
//
// Revision 1.7  1998/07/03 16:27:17  tom
// MailboxInspector:
// (-revert:): don't enable incorporate button if mailbox is not loaded.
// (#ICON_NEWMAIL,#ICON_INVALID): new string constants.
// (-load): show counts regardless of mailbox validity--trust MailboxSummary
//  to do the right thing; invert sense of pendingAppnmailIconButton and
//  indexTagButton state, so icons show up in IB by default; set altIcon of
//  newMailIconButton according to validity, to either ICON_NEWMAIL or
//  ICON_INVALID and set its state accordingly.
//
// Revision 1.6  1998/04/28 15:13:48  tom
// MailboxInspector:
// - overhauled to use MailboxSummary:
// (mailbox): now instance of EnhanceMailboxSummary.
// (stats): ivar removed.
// (-shouldLoad,-getPath:forFile:): methods removed.
// (-load): now returns void.
// - check for pending appnmail:
// (pendingAppnmailIconButton): new outlet.
// (#INCORPBUTTON): new localizable constant.
// (-revert:): handle revertButton: incorporate pending appnmail.
//
// PB.project Makefile:
// (Classes): add MailboxSummary.
//
// Revision 1.5  1997/05/10 15:22:21  tom
// MailboxInspector:
// (tocSizeField): new outlet.
// (VERSION): bumped to 1.1.
// (TOC_SIZE): new localizable string.
// (LOCALIZE): hack to keep "genstrings" from breaking.
// (-load): show toc size.
//
// Makefile.preamble:
// (GENSTRINGS,LANGUAGE_DIRECTORIES): new variables.
//
// Makefile.postamble:
// (clean::,resources::,genstrings:): new targets.
//
// README.rtf:
// - updated for 1.1.
//
// Revision 1.4  1997/04/19 18:14:56  tom
// (VERSION): upped to 1.0.
// (-load): adapt to mailtoc API changes.
//
// Revision 1.3  1996/06/27 22:55:52  tom
// Add sccs `what' marker in ident.
//
// Revision 1.2  1996/06/27 20:04:40  tom
// (VERSION): bumped to 0.9.
// (INDEX_SIZE): new localizable.
// (-load): show size of index store.
//
// Revision 1.1  1996/06/26 14:40:42  tom
// Initial revision
//
//======================================================================
